import jinja2
from jingo import register

from .utils import (urlize_mentions as do_urlize_mentions, get_mentions,
                    plain_mentions as do_plain_mentions)


@register.filter
def urlize_mentions(text, prefetched=None, urlize=None):
    rv = do_urlize_mentions(text, prefetched=prefetched, urlize=urlize)
    return jinja2.Markup(rv)


@register.filter
def plain_mentions(text):
    return do_plain_mentions(text)


@register.function
def prefetch_mentions(object_list, attr):
    texts = [getattr(obj, attr) for obj in object_list]
    return get_mentions(''.join(texts))



from django.contrib.auth import get_user_model

from .registry import Provider


class UserProvider(Provider):
    model = get_user_model()

    def get_title(self, obj):
        return obj.username

    def search(self, request, term):
        return self.get_queryset().filter(username__istartswith=term)



import re
from importlib import import_module

from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured
from django.utils.functional import lazy
from django.core import exceptions
from django.conf import settings


class Provider(object):
    model = None

    def __init__(self):
        self.name = re.sub('provider$', '', self.__class__.__name__.lower())

    def get_queryset(self):
        if self.model is None:
            raise ImproperlyConfigured('No model defined')
        return self.model._default_manager.all()

    def get_object(self, pk):
        try:
            return self.get_queryset().get(pk=pk)
        except ObjectDoesNotExist:
            return None

    def search(self, request, term):
        raise NotImplemented

    def get_title(self, obj):
        raise NotImplemented

    def get_image(self, obj):
        return None


class ProviderRegistry(object):
    def __init__(self):
        self.providers = set()
        self.collections = {}

    def register(self, provider, collection='default'):
        provider = provider()
        self.providers.add(provider)
        self.collections.setdefault(collection, []).append(provider)

    def for_name(self, name):
        return next(p for p in self.providers if p.name == name)

    def for_model(self, model):
        model = model._meta.concrete_model
        return next(p for p in self.providers if p.model == model)

    def get_collection(self, name):
        return self.collections[name]


def _bootstrap():
    _registry = ProviderRegistry()
    MENTIONED_CLASSES = getattr(settings, 'MENTIONS_PROVIDERS', {})
    for collection, path_list in MENTIONED_CLASSES.items():
        for path in path_list:
            try:
                module, classname = path.rsplit('.', 1)
            except ValueError:
                raise exceptions.ImproperlyConfigured('%s isn\'t a module' % path)
            try:
                mod = import_module(module)
            except ImportError, e:
                raise exceptions.ImproperlyConfigured('Error importing class %s: "%s"' % (module, e))
            try:
                cls = getattr(mod, classname)
            except AttributeError:
                raise exceptions.ImproperlyConfigured('Module "%s" does not define a "%s" class' % (module, classname))
            _registry.register(cls, collection)
    return _registry


providers = lazy(_bootstrap, ProviderRegistry)()



import json

from django.http import HttpResponse


from .registry import providers


def autocomplete(request):
    term = request.GET.get('term', '')
    collection_name = request.GET.get('mentions', 'default')

    ret = []
    if term:
        try:
            collection = providers.get_collection(collection_name)
        except KeyError:
            collection = providers.get_collection('default')

        for provider in collection:
            objs = provider.search(request, term)
            for obj in objs:
                ret.append({
                    'value': provider.get_title(obj),
                    'image': provider.get_image(obj),
                    'uid': '%s:%s' % (provider.name, obj.pk),
                })

    return HttpResponse(json.dumps(ret), mimetype='application/json')



import json
from django.test import TestCase
from django.db import models
from django import forms
from django.core.urlresolvers import reverse
from mock import patch

from .models import MentionTextField
from .registry import Provider, ProviderRegistry
from .utils import make_mention
from .helpers import urlize_mentions
from .forms import MentionTextarea


class User(models.Model):
    name = models.CharField(max_length=30)

    def get_absolute_url(self):
        return '/users/%d/' % self.pk


class Post(models.Model):
    text = MentionTextField(links=['users'])

    users = models.ManyToManyField(User)


class UserProvider(Provider):
    model = User

    def get_title(self, obj):
        return obj.name

    def search(self, request, term):
        return self.get_queryset().filter(name__istartswith=term)


providers = ProviderRegistry()
providers.register(UserProvider)


@patch('mentions.utils.providers', providers)
@patch('mentions.models.providers', providers)
@patch('mentions.views.providers', providers)
class MentionTest(TestCase):
    def setUp(self):
        self.user = User.objects.create(pk=1, name='Johny')

    def test_filter(self):
        self.text = 'I hate you %s' % make_mention(self.user)
        self.assertEqual(urlize_mentions(self.text),
                         'I hate you <a href="/users/1/">Johny</a>')

    def test_link(self):
        post = Post.objects.create(text=make_mention(self.user))
        self.assertEqual(post.users.get(), self.user)

    def test_widget(self):
        class PostForm(forms.ModelForm):
            class Meta:
                model = Post
        form = PostForm()
        self.assertEqual(form.fields['text'].widget.__class__, MentionTextarea)
        unicode(form.fields['text'])

    def test_widget_render(self):
        MentionTextarea().render(name='text', value='sosiska')

    def test_view(self):
        User.objects.create(name='Kenny')
        response = self.client.get(reverse('mention_complete'), data={'term': 'j'})
        content = json.loads(response.content)
        expected = [{'value': 'Johny', 'image': None, 'uid': 'user:1'}]
        self.assertEqual(content, expected)



import uuid

from django import forms
from django.core.urlresolvers import reverse


class MentionTextarea(forms.Textarea):
    """Required: jquery"""
    class Media:
        css = {
            'all': ('css/jquery.mentions.css',)
        }
        js = ('js/jquery.mentions.js',)

    def render(self, name, value, attrs=None):
        extra_attrs = {
            'id': uuid.uuid4().hex,
            'data-source': reverse('mention_complete')
        }
        if attrs:
            extra_attrs.update(attrs)
        return super(MentionTextarea, self).render(name, value, extra_attrs)



# coding: utf-8
from functools import partial

from django.db.models.signals import post_save, post_init
from django.db import models
from django.conf import settings

from .forms import MentionTextarea
from .registry import providers
from .utils import get_mentions
from .signals import objects_mentioned


class AddListener(object):
    def __init__(self, field):
        self.field = field
        self.name = '_' + field.name

    def contribute_to_class(self, cls, name):
        register_listener(cls, self.field)


class MentionTextField(models.TextField):
    def __init__(self, *args, **kwargs):
        self.links = kwargs.pop('links', [])
        super(MentionTextField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'widget': MentionTextarea()}
        defaults.update(**kwargs)
        return super(MentionTextField, self).formfield(**defaults)

    def contribute_to_class(self, cls, name):
        super(MentionTextField, self).contribute_to_class(cls, name)
        if not cls._meta.abstract and not cls._meta.swapped:
            register_listener(cls, self)
            # Черствый хак, для того чтобы contribute_to_class вызывался и
            # для дочерних моделей
            cls._meta.add_virtual_field(AddListener(self))


def register_listener(cls, field):
    process = partial(process_mentions, field=field)
    post_save.connect(process, sender=cls, weak=False)
    post_init.connect(partial(preserve_orig_values, field=field),
                      sender=cls, weak=False)


def preserve_orig_values(instance, field, **kwargs):
    setattr(instance, '_%s_initial' % field.name,
            field.value_to_string(instance))


def process_mentions(instance, created, field, **kwargs):
    initial = getattr(instance, '_%s_initial' % field.name)
    text = field.value_to_string(instance)

    if not created and initial == text:
        return

    m2mfields = dict((f.rel.to, f) for f in instance._meta.many_to_many
                     if f.name in field.links)
    for provider_name, objs in get_mentions(text).items():
        try:
            get_extra_mentions = getattr(instance, 'get_%s_mentions' % provider_name)
        except AttributeError:
            pass
        else:
            objs = set(objs) | set(get_extra_mentions())

        model = providers.for_name(provider_name).model
        if model in m2mfields:
            setattr(instance, m2mfields[model].name, objs)
        objects_mentioned.send(mentions=objs, instance=instance, created=created, sender=model)


if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([
        (
            [MentionTextField],
            [],
            {
                #'links': ['links', {}],
            }
        ),
    ], ["^mentions\.models\.MentionTextField"])


from setuptools import setup, find_packages

setup(
    name='django-mentions',
    version='0.1.8',
    author='Ivan Virabyan',
    packages=find_packages(),
    description='Facebook-like mentions for Django',
    include_package_data=True,
)



import re
from .registry import providers

MENTION_PATTERN = re.compile('@\[(?P<title>[^\]]+)\]\((\w+):(\d+)\)')


def urlize_mentions(text, prefetched=None, urlize=None):
    if urlize is None:
        urlize = lambda url, title: '<a href="%s">%s</a>' % (url, title)

    def insert_links(m):
        raw, provider_name, pk = m.groups()
        try:
            pk = int(pk)
            obj = prefetched[provider_name][pk]
        except (ValueError, KeyError):
            return raw
        provider = providers.for_name(provider_name)
        return urlize(obj.get_absolute_url(), provider.get_title(obj))

    if prefetched is None:
        prefetched = get_mentions(text)

    prefetched = dict((provider_name, dict((obj.pk, obj) for obj in objs))
                      for provider_name, objs in prefetched.items())
    return MENTION_PATTERN.sub(insert_links, unicode(text))


def plain_mentions(text):
    return MENTION_PATTERN.sub('\g<title>', text)


def make_mention(obj):
    provider = providers.for_model(obj)
    return '@[%s](%s:%s)' % (provider.get_title(obj), provider.name, obj.pk)


def get_mentions(text):
    mentions = [(type_, pk) for raw, type_, pk in MENTION_PATTERN.findall(unicode(text))]

    by_type = {}
    for type_, pk in mentions:
        by_type.setdefault(type_, set()).add(pk)

    ret = {}
    for provider_name, pks in by_type.items():
        provider = providers.for_name(provider_name)
        ret[provider_name] = provider.get_queryset().filter(pk__in=pks)
    return ret
